#ifdef HAVE_GLX
		OGL_CONST_i(GLX_USE_GL)
		OGL_CONST_i(GLX_BUFFER_SIZE)
		OGL_CONST_i(GLX_LEVEL)
		OGL_CONST_i(GLX_RGBA)
		OGL_CONST_i(GLX_DOUBLEBUFFER)
		OGL_CONST_i(GLX_STEREO)
		OGL_CONST_i(GLX_AUX_BUFFERS)
		OGL_CONST_i(GLX_RED_SIZE)
		OGL_CONST_i(GLX_GREEN_SIZE)
		OGL_CONST_i(GLX_BLUE_SIZE)
		OGL_CONST_i(GLX_ALPHA_SIZE)
		OGL_CONST_i(GLX_DEPTH_SIZE)
		OGL_CONST_i(GLX_STENCIL_SIZE)
		OGL_CONST_i(GLX_ACCUM_RED_SIZE)
		OGL_CONST_i(GLX_ACCUM_GREEN_SIZE)
		OGL_CONST_i(GLX_ACCUM_BLUE_SIZE)
		OGL_CONST_i(GLX_ACCUM_ALPHA_SIZE)
#ifdef GLX_EXT_visual_info
		OGL_CONST_i(GLX_X_VISUAL_TYPE_EXT)
		OGL_CONST_i(GLX_TRANSPARENT_TYPE_EXT)
		OGL_CONST_i(GLX_TRANSPARENT_INDEX_VALUE_EXT)
		OGL_CONST_i(GLX_TRANSPARENT_RED_VALUE_EXT)
		OGL_CONST_i(GLX_TRANSPARENT_GREEN_VALUE_EXT)
		OGL_CONST_i(GLX_TRANSPARENT_BLUE_VALUE_EXT)
		OGL_CONST_i(GLX_TRANSPARENT_ALPHA_VALUE_EXT)
#endif
		OGL_CONST_i(X_PROTOCOL)
		OGL_CONST_i(X_PROTOCOL_REVISION)
		OGL_CONST_i(NoEventMask)
		OGL_CONST_i(KeyPressMask)
		OGL_CONST_i(KeyReleaseMask)
		OGL_CONST_i(ButtonPressMask)
		OGL_CONST_i(ButtonReleaseMask)
		OGL_CONST_i(EnterWindowMask)
		OGL_CONST_i(LeaveWindowMask)
		OGL_CONST_i(PointerMotionMask)
		OGL_CONST_i(PointerMotionHintMask)
		OGL_CONST_i(Button1MotionMask)
		OGL_CONST_i(Button2MotionMask)
		OGL_CONST_i(Button3MotionMask)
		OGL_CONST_i(Button4MotionMask)
		OGL_CONST_i(Button5MotionMask)
		OGL_CONST_i(ButtonMotionMask)
		OGL_CONST_i(KeymapStateMask)
		OGL_CONST_i(ExposureMask)
		OGL_CONST_i(VisibilityChangeMask)
		OGL_CONST_i(StructureNotifyMask)
		OGL_CONST_i(ResizeRedirectMask)
		OGL_CONST_i(SubstructureNotifyMask)
		OGL_CONST_i(SubstructureRedirectMask)
		OGL_CONST_i(FocusChangeMask)
		OGL_CONST_i(PropertyChangeMask)
		OGL_CONST_i(ColormapChangeMask)
		OGL_CONST_i(OwnerGrabButtonMask)
		OGL_CONST_i(KeyPress)
		OGL_CONST_i(KeyRelease)
		OGL_CONST_i(ButtonPress)
		OGL_CONST_i(ButtonRelease)
		OGL_CONST_i(MotionNotify)
		OGL_CONST_i(EnterNotify)
		OGL_CONST_i(LeaveNotify)
		OGL_CONST_i(FocusIn)
		OGL_CONST_i(FocusOut)
		OGL_CONST_i(KeymapNotify)
		OGL_CONST_i(Expose)
		OGL_CONST_i(GraphicsExpose)
		OGL_CONST_i(NoExpose)
		OGL_CONST_i(VisibilityNotify)
		OGL_CONST_i(CreateNotify)
		OGL_CONST_i(DestroyNotify)
		OGL_CONST_i(UnmapNotify)
		OGL_CONST_i(MapNotify)
		OGL_CONST_i(MapRequest)
		OGL_CONST_i(ReparentNotify)
		OGL_CONST_i(ConfigureNotify)
		OGL_CONST_i(ConfigureRequest)
		OGL_CONST_i(GravityNotify)
		OGL_CONST_i(ResizeRequest)
		OGL_CONST_i(CirculateNotify)
		OGL_CONST_i(CirculateRequest)
		OGL_CONST_i(PropertyNotify)
		OGL_CONST_i(SelectionClear)
		OGL_CONST_i(SelectionRequest)
		OGL_CONST_i(SelectionNotify)
		OGL_CONST_i(ColormapNotify)
		OGL_CONST_i(ClientMessage)
		OGL_CONST_i(MappingNotify)
		OGL_CONST_i(LASTEvent)
		OGL_CONST_i(ShiftMask)
		OGL_CONST_i(LockMask)
		OGL_CONST_i(ControlMask)
		OGL_CONST_i(Mod1Mask)
		OGL_CONST_i(Mod2Mask)
		OGL_CONST_i(Mod3Mask)
		OGL_CONST_i(Mod4Mask)
		OGL_CONST_i(Mod5Mask)
		OGL_CONST_i(ShiftMapIndex)
		OGL_CONST_i(LockMapIndex)
		OGL_CONST_i(ControlMapIndex)
		OGL_CONST_i(Mod1MapIndex)
		OGL_CONST_i(Mod2MapIndex)
		OGL_CONST_i(Mod3MapIndex)
		OGL_CONST_i(Mod4MapIndex)
		OGL_CONST_i(Mod5MapIndex)
		OGL_CONST_i(Button1Mask)
		OGL_CONST_i(Button2Mask)
		OGL_CONST_i(Button3Mask)
		OGL_CONST_i(Button4Mask)
		OGL_CONST_i(Button5Mask)
		OGL_CONST_i(AnyModifier)
		OGL_CONST_i(Button1)
		OGL_CONST_i(Button2)
		OGL_CONST_i(Button3)
		OGL_CONST_i(Button4)
		OGL_CONST_i(Button5)
		OGL_CONST_i(NotifyNormal)
		OGL_CONST_i(NotifyGrab)
		OGL_CONST_i(NotifyUngrab)
		OGL_CONST_i(NotifyWhileGrabbed)
		OGL_CONST_i(NotifyHint)
		OGL_CONST_i(NotifyAncestor)
		OGL_CONST_i(NotifyVirtual)
		OGL_CONST_i(NotifyInferior)
		OGL_CONST_i(NotifyNonlinear)
		OGL_CONST_i(NotifyNonlinearVirtual)
		OGL_CONST_i(NotifyPointer)
		OGL_CONST_i(NotifyPointerRoot)
		OGL_CONST_i(NotifyDetailNone)
		OGL_CONST_i(VisibilityUnobscured)
		OGL_CONST_i(VisibilityPartiallyObscured)
		OGL_CONST_i(VisibilityFullyObscured)
		OGL_CONST_i(PlaceOnTop)
		OGL_CONST_i(PlaceOnBottom)
		OGL_CONST_i(FamilyInternet)
		OGL_CONST_i(FamilyDECnet)
		OGL_CONST_i(FamilyChaos)
		OGL_CONST_i(PropertyNewValue)
		OGL_CONST_i(PropertyDelete)
		OGL_CONST_i(ColormapUninstalled)
		OGL_CONST_i(ColormapInstalled)
		OGL_CONST_i(GrabModeSync)
		OGL_CONST_i(GrabModeAsync)
		OGL_CONST_i(GrabSuccess)
		OGL_CONST_i(AlreadyGrabbed)
		OGL_CONST_i(GrabInvalidTime)
		OGL_CONST_i(GrabNotViewable)
		OGL_CONST_i(GrabFrozen)
		OGL_CONST_i(AsyncPointer)
		OGL_CONST_i(SyncPointer)
		OGL_CONST_i(ReplayPointer)
		OGL_CONST_i(AsyncKeyboard)
		OGL_CONST_i(SyncKeyboard)
		OGL_CONST_i(ReplayKeyboard)
		OGL_CONST_i(AsyncBoth)
		OGL_CONST_i(SyncBoth)
		OGL_CONST_i(StaticGray)
		OGL_CONST_i(GrayScale)
		OGL_CONST_i(StaticColor)
		OGL_CONST_i(PseudoColor)
		OGL_CONST_i(TrueColor)
		OGL_CONST_i(DirectColor)
#endif /* def HAVE_GLX */
